%%Inference.pl
%%Contains the inference engine of the prototype
%%User commands are:
%%make_horn/2
%%inference/0
%%retract_horn/1

%%make_horn(+Horns, +Head, +J)
%%Creates a new horn clause Horns=>Head using J as a justification
%% Only create a new node if this one isn't in the knowledge base
make_horn(Horns, Head, J):-	\+ horn(Horns, Head),
				write('adding '), write(Horns), write('=>'), write(Head), write('\n'),
				jtms_create_node(Head, J),
				assert(horn(Horns, Head)).

%%make_horn(+Horns, +Head)
%%Creates a new horn clause Horns=>Head with no justifications (ie an assumption or atomic fact)
make_horn(Horns, Head):-	\+ horn(Horns, Head),
				write('adding '), write(Horns), write('=>'), write(Head), write('\n'),
				jtms_create_node(Head, Horns),
				assert(horn(Horns, Head)).


%%modus_ponens(+HornClause)
%%perform modus ponens on a horn clause
modus_ponens(horn(Justifications, Head)):- modus_ponens(horn(Justifications,Head), Justifications).
modus_ponens(horn([], Head), J):-	make_horn([true], Head, J).
modus_ponens(horn([H|T], Head), J):-	horn([true], H),
					modus_ponens(horn(T, Head), J).

%%inference
%%find all the horn clauses in the database and perform modus ponens on them
inference:-	infer, inference.
inference.

%%infer 
%%intermediate step during predicate inference finds all of the horn clauses in the db
infer:-		findall(horn(X,Y), horn(X,Y), Bag),
			do_infer(Bag, false).

%%do_infer(+Clauses, ?flag)
%%performs one step of modus ponens, sets flag to true if at least one
%inference has been made
do_infer([], true).
do_infer([], false):- fail.
do_infer([H|T], _):-	modus_ponens(H),
						do_infer(T, true).
do_infer([_|T], Flag):-	do_infer(T, Flag).

%%retract_horn(+HornClause)
%%Removes a horn clause from the knowledgebase
retract_horn(horn(Horns, Head)):-	horn(Horns, Head),
					jtms_retract(Head),
					retract(horn(Horns, Head)).

%%retract_all(+List)
%%Convenience predicate which retracts all horn clauses found in list
retract_all([]).
retract_all([H|T]):-	findall(horn(A, H), horn(A, H), Bag), 
						retract_all_horns(Bag), 
						retract_all(T).

%%retract_all_horns(+HornClauseList)
%%part of retract_all/1
retract_all_horns([]).
retract_all_horns([H|T]):- retract_horn(H), retract_all_horns(T).
					